
/*
*	timing.c
*
*	Does a few communication timing tests on pvm.
*	Uses `timing_slave' to echo messages.
* ----------------------------------------------------------------------------
*     If this test is run over machines with differnet data formats
*     Then change 'ENCODING' to PvmDataDefault in timing and timing_slave
* ----------------------------------------------------------------------------
*
*	9 Dec 1991  Manchek
*  14 Oct 1992  Geist  - revision to pvm3
*   6 Mar 1994  Geist  - synch tasks and add direct route
*/

#include <stdio.h>
#include <sys/time.h>
#include <time.h>

#include <sys/types.h>
#include <math.h>
#include "timing.h"


/**** Returns size of file *************/
int read_pict(char *argv, char* image, int stid)
{
  FILE *input;
  int i=-1;
  int temp;
  if ((input = fopen(argv, "r")) == NULL){
    printf("Error opening input file\n");
    pvm_kill(stid);
    pvm_exit();
    exit(1);
  }
  while ((temp=fgetc(input)) != EOF)
    {
      i++;
      image[i]=temp;
    }
  fclose(input);
  return i;
}


void send_slaves(int stid, char** argv)
{
  int tids[NTIDS];
  
  if (pvm_spawn(OTHERSLAVE, argv, 2, "ALPHA", NTIDS, tids) < NTIDS) {
    fputs("can't initiate slaves\n", stderr);
    exit(1);
  }
  
  /**** Broadcast where to send data ************/
  pvm_initsend(ENCODING);
  pvm_pkint(&stid, 1, 1);
  pvm_pkint(tids, NTIDS, 1);
  pvm_mcast(tids, NTIDS, MSGTAG);
}


main(int argc, char **argv)
{
  int mytid;                    /* my task id */
  int stid = 0;			/* slave task id */
  int reps = 20;		/* number of samples per test */
  struct timeval tv1, tv2;	/* for timing */
  int dt1, dt2;			/* time for one iter */
  int at1, at2;			/* accum. time */
  int numint;	       		/* message length */
  char image[921615];           /* picture to send */
  int n;
  int i, done;
  
  /* enroll in pvm */
  
  if ((mytid = pvm_mytid()) < 0) {
    exit(1);
  }
  printf("i'm t%x\n", mytid);
  
  /* start up slave task */
  
  if (pvm_spawn(SLAVENAME, (char**)0, 1, "kopernik-fddi", 1, &stid) < 0 || stid < 0) {
		fputs("can't initiate slave\n", stderr);
		goto bail;
	      }
  

  /* Wait for slave task to start up */
  /*    pvm_setopt(PvmRoute, PvmRouteDirect);  This causes megaerrors */
  pvm_recv( stid, 0);
  printf("slave is task t%x\n", stid);
  
  /*
   *  single message timing test
   */
  
  numint = read_pict(argv[1], image, stid);
  
  printf("\nMessage size %d\n", numint);
  at1 = at2 = 0;
  puts(" N  Pack uSec  Send uSec\n");
  for (n = 1; n <= reps; n++) {
    gettimeofday(&tv1, (struct timezone*)0);
    
    pvm_initsend(ENCODING);
    pvm_pkbyte(image, numint, 1);
    
    gettimeofday(&tv2, (struct timezone*)0);
    dt1 = (tv2.tv_sec - tv1.tv_sec) * 1000000
      + tv2.tv_usec - tv1.tv_usec;
    
    gettimeofday(&tv1, (struct timezone*)0);
    
    if (pvm_send(stid, 1)) {
      fprintf(stderr, "can't send to \"%s\"\n", SLAVENAME);
      goto bail;
    }
    
    if (pvm_recv(-1, -1) < 0) {
      fprintf(stderr, "recv error%d\n" );
      goto bail;
    }
    
    gettimeofday(&tv2, (struct timezone*)0);
      dt2 = (tv2.tv_sec - tv1.tv_sec) * 1000000
	+ tv2.tv_usec - tv1.tv_usec;
    
    printf("%2d   %8d   %8d\n", n, dt1, dt2);
    at1 += dt1;
    at2 += dt2;
  }
  
  if (!(at1 /= reps))
    at1 = 1;
  if (!(at2 /= reps))
    at2 = 1;
  puts("Avg uSec");
  printf("     %8d   %8d\n", at1, at2);
  puts("Avg Byte/uSec");
    printf("     %8f   %8f\n",
	   (numint) / (double)at1,
	   (numint) / (double)at2);
  
  
  puts("\ndone");
  
  gettimeofday(&tv1, (struct timezone*)0);
  send_slaves(stid, argv);
  
  for (i=0; i<NTIDS; i++)
    {
      pvm_recv(-1,-1);
      pvm_upkint(&done, 1, 1);
    }
  gettimeofday(&tv2, (struct timezone*)0);
  printf(" Elapsed time %f\n",(tv2.tv_sec - tv1.tv_sec) * 1000000
	+ tv2.tv_usec - tv1.tv_usec);

 goto bail;
 bail:
  printf("Stid: %d\n", stid);
  if (stid > 0)
    if (pvm_kill(stid) <0)
      printf("Error killing slave.\n");
  pvm_exit();
  exit(1);
}



